@@ -5,14 +5,12 @@ from taped.base import (
     ByteChunks,
     WfChunks,
     LiveWf,
-    BufferItemOutput
+    BufferItemOutput,
 )
 from taped.util import (
     list_recording_device_index_names,
     find_a_default_input_device_index,
     simple_chunker,
-    rechunker
+    rechunker,
+    chunk_indices,
 )
-
-
-
